% @author: Wenqiang Yan

% This code is used for the time-frequency images fusion

clc
close all
clear all
feature jit off % Accelerate code run
DATA = load('...\MATLAB Code\data_test.mat'); %Import mate file format: Sampling points*channel*trial
Fs=1200;% sampling frequency
nn=5; %Stimulus duration of each trial
t = (0:1/Fs:(nn*Fs-1)/Fs)';
NN=4; % the number of layers of wavelet decomposition
fps=[7 8 9 10 7.2 8.2 9.2 10.2 7.4 8.4 9.4 10.4 7.6 8.6 9.6 10.6 7.8 8.8 9.8 10.8]; % stimulus frequency
KK=10; % the template size of the mean filtering
MM=[1 2]; % The first and second channel signals are used in T-F image fusion

for i=1:20 % 20 targets
    for k=1:2
        a=DATA.data(:,MM(k),i);
        aa=detrend(a,'constant');
        aaa=smooth(aa,30,'loess');
        b(:,k)=bandp(aaa,0.5,50,0.2,60,Fs); %Chebyshev band pass filtering
        
        % Short time Fourier transform
        h=gausswin(57); % the Gaussian window length 57
        [tfr(:,:,k),T,F]=tfrstft(10^6*b(:,k),1:length(b),54,h,1); % the number of frequency bins 54
        
        % Two-dimensional multi-scale wavelet decomposition
        [c(:,:,k),l]=wavedec2(tfr(:,:,k),NN,'db2');
        A(:,:,k)=appcoef2(c(:,:,k),l,'db2');
        [chd1(:,:,k),cvd1(:,:,k),cdd1(:,:,k)] = detcoef2('all',c(:,:,k),l,1);
        [chd2(:,:,k),cvd2(:,:,k),cdd2(:,:,k)] = detcoef2('all',c(:,:,k),l,2);
        [chd3(:,:,k),cvd3(:,:,k),cdd3(:,:,k)] = detcoef2('all',c(:,:,k),l,3);
        [chd4(:,:,k),cvd4(:,:,k),cdd4(:,:,k)] = detcoef2('all',c(:,:,k),l,4);
    end
    
    % Low frequency wavelet component fusion
    A0=A(:,:,1)-A(:,:,2); % You can choose A(:,:,1)-A(:,:,2) to change the value of LLN,F
    
    % High frequency wavelet component fusion
    for ii=1:size(cdd1,1)
        for jj=1:size(cdd1,2)
            [~,T1]=max([real(chd1(ii,jj,1)) real(chd1(ii,jj,2))]);
            [~,T2]=max([real(cvd1(ii,jj,1)) real(cvd1(ii,jj,2))]);
            [~,T3]=max([real(cdd1(ii,jj,1)) real(cdd1(ii,jj,2))]);
            Chd1(ii,jj)=chd1(ii,jj,T1);
            Cvd1(ii,jj)=cvd1(ii,jj,T2);
            Cdd1(ii,jj)=cdd1(ii,jj,T3);
        end
    end
    for ii=1:size(cdd2,1)
        for jj=1:size(cdd2,2)
            [~,T1]=max([real(chd2(ii,jj,1)) real(chd2(ii,jj,2))]);
            [~,T2]=max([real(cvd2(ii,jj,1)) real(cvd2(ii,jj,2))]);
            [~,T3]=max([real(cdd2(ii,jj,1)) real(cdd2(ii,jj,2))]);
            Chd2(ii,jj)=chd2(ii,jj,T1);
            Cvd2(ii,jj)=cvd2(ii,jj,T2);
            Cdd2(ii,jj)=cdd2(ii,jj,T3);
        end
    end
    for ii=1:size(cdd3,1)
        for jj=1:size(cdd3,2)
            [~,T1]=max([real(chd3(ii,jj,1)) real(chd3(ii,jj,2))]);
            [~,T2]=max([real(cvd3(ii,jj,1)) real(cvd3(ii,jj,2))]);
            [~,T3]=max([real(cdd3(ii,jj,1)) real(cdd3(ii,jj,2))]);
            Chd3(ii,jj)=chd3(ii,jj,T1);
            Cvd3(ii,jj)=cvd3(ii,jj,T2);
            Cdd3(ii,jj)=cdd3(ii,jj,T3);
        end
    end
    for ii=1:size(cdd4,1)
        for jj=1:size(cdd4,2)
            [~,T1]=max([real(chd4(ii,jj,1)) real(chd4(ii,jj,2))]);
            [~,T2]=max([real(cvd4(ii,jj,1)) real(cvd4(ii,jj,2))]);
            [~,T3]=max([real(cdd4(ii,jj,1)) real(cdd4(ii,jj,2))]);
            Chd4(ii,jj)=chd4(ii,jj,T1);
            Cvd4(ii,jj)=cvd4(ii,jj,T2);
            Cdd4(ii,jj)=cdd4(ii,jj,T3);
        end
    end
    
    % Obtain the wavelet high frequency component and low frequency component of the fused image
    A00=reshape(A0,1,size(A,1)*size(A,2));
    C_1=reshape([Chd1 Cvd1 Cdd1],1,3*size(cdd1,1)*size(cdd1,2));
    C_2=reshape([Chd2 Cvd2 Cdd2],1,3*size(cdd2,1)*size(cdd2,2));
    C_3=reshape([Chd3 Cvd3 Cdd3],1,3*size(cdd3,1)*size(cdd3,2));
    C_4=reshape([Chd4 Cvd4 Cdd4],1,3*size(cdd4,1)*size(cdd4,2));
    C=[A00 C_4 C_3 C_2 C_1];
    
    % Two-dimensional wavelet reconstruction
    X2 = waverec2(C,l,'db2');
    
    % Mean filtering
    X3 = filter2(fspecial('average',KK),X2)/255;%  the template size of the mean filtering KK
    
    % inverse STFT
    sigsyn=tfristft(X3,1:length(b),h,1);
    
    % Welch power spectrum
    Nfft=nn*Fs;
    noverlap=256;
    dflag='onesided';
    window=gausswin(nn*Fs);
    [Pxx1,f1]=pwelch(real(sigsyn),window,noverlap,Nfft,Fs,dflag);
    
    % Plotting Welch power spectrum
    subplot(5,4,i)
    plot(f1(1:Nfft/30),Pxx1(1:Nfft/30),'b','LineWidth',1.2);
    hold on
    plot(f1(nn*fps(i)+1),Pxx1(nn*fps(i)+1), 'o','LineWidth',1.5,'MarkerSize',6,'MarkerEdgeColor','r');
    
    % Identify the target frequency
    for ii=1:length(fps)
        k(ii)=Pxx1(nn*fps(ii)+1,1);
    end
    [c1,c2(1,i)]=max(k); % The frequency corresponding to the highest amplitude is identified as the focused target frequency
    
end
KKK=1:1:20;
Number=length(find((c2-KKK)==0)) % Output the number of targets that were correctly identified



